#<profile4xi profile = "03" />
#<program4xi program = "pipeconnection4xi.py">
#<abstract4xi abstract = "providing class PipeConnection4Xi"/>
#<refer4xi reference = "connection.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'PipeConnection4Xi']
#"
#/>
#

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import io
import os
import sys
import pickle
import select
import socket
import struct
import errno
import time
import tempfile
import itertools
try:
    import _winapi
    from _winapi import WAIT_OBJECT_0, WAIT_TIMEOUT, INFINITE
except ImportError:
    if sys.platform == 'win32':
        raise
    _winapi = None
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
from multiprocessing import connection 
from multiprocessing import reduction

try:                             # Python ones
    from multiprocessing.connection import PipeConnection
except:
    print("pipeconnection4xi.py *** failed in from multiprocessing.connection import PipeConnection ***") 

#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from forkingpickler4xi import ForkingPickler4Xi

from current_process4xi import get_authkey_current_process4xi

from register_after_fork4xi import register_after_fork4xi

from sub_warning4xi import sub_warning4xi

from timeout4xi import omega2timeout4xi
from print4xi import print4xi
from time4xi import time4xi    
from wait4xi import wait4xi

from _connectionbase4xi import _ConnectionBase4Xi
#"
#/>

# =================================================================================================
#<class4xi class = "PipeConnection4Xi" base = "object" child = "None" synopsis = "PipeConnection4Xi class" usage = "
#   connection.py
#       if sys.platform != 'win32':
#           ...
#       else:
#           def Pipe(duplex=True):
#               ...
#               c1 = PipeConnection(h1, writable=duplex)
#               c2 = PipeConnection(h2, readable=duplex)
#               return c1, c2
#       ...
#       if sys.platform == 'win32':
#           class PipeListener(object):
#               ...
#               def accept(self):
#                   ...
#                   return PipeConnection(handle)
#           def PipeClient(address):
#               ...
#               return PipeConnection(h)
#   pipe4xi.py
#       def Pipe4Xi(duplex=True):
#           ...
#           c1 = PipeConnection4Xi(h1, writable=duplex)
#           c2 = PipeConnection4Xi(h2, readable=duplex)
#           return c1, c2
#   pipelistener4xi.py
#       class PipeListener4Xi(object):
#           ...
#           def accept(self):
#               ...
#               return PipeConnection4Xi(handle)
#   pipeclient4xi.py
#       def PipeClient4Xi(address):
#           ...
#           return PipeConnection4Xi(h)
#>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# =================================================================================================
class PipeConnection4Xi(_ConnectionBase4Xi):
    '''
    PipeConnection4Xi class based on a Windows named pipe using Overlapped I/O
    which means handles must have been created with FILE_FLAG_OVERLAPPED
    '''

    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = ('sense', 'reason', 'ace')          
    #"
    #/>    
     #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    _got_empty_message = False
    #" 
    #/>
            
    # JATKA LISAA ORIGINAL JA SITEN __INIT__
    
            
    # ------------------------------------------------------------------------------------------------------------------------------------
    #<methodize4xi method = "close-of-PipeConnection4Xi" usage = "closes PipeConnection4Xi with (self, _CloseHandle=_winapi.CloseHandle)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ------------------------------------------------------------------------------------------------------------------------------------
    def _close(self, _CloseHandle=_winapi.CloseHandle):
        '''
        closes PipeConnection4Xi
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/2)            
        # -----------------------------------------------------------------------------------------
        class2print = "PipeConnection4Xi"
        method2print = "_close"
        
        message2print = "{(Step 1/2a) started" + "; close-of-PipeConnection4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 05:22:44 01/08/13 Pacific Standard Time
        
        message2print = "{(Step 1/2b) passed" + "; CloseHandle-to-close-of-PipeConnection4Xi: " + str(_CloseHandle) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # <built-in function CloseHandle> 
        # <built-in function CloseHandle>
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/2)            
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 2/2a) passing" + "; handle-passing-to-CloseHandle-by-close-of-PipeConnection4Xi: " + str(self._handle) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 248; 256
        
        try:
            _CloseHandle(self._handle)
            
            message2print = "{(Step 2/2b) performed" + "; CloseHandle-performed-by-close-of-PipeConnection4Xi: " + str(_CloseHandle)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <built-in function CloseHandle>
            # <built-in function CloseHandle>
        except:
            message2print = "{(Step 2/2z) failed" + "; CloseHandle-failed-by-close-of-PipeConnection4Xi: " + str(_CloseHandle)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
        
        message2print = "{(Step 2/2c) returning" + "; close-of-PipeConnection4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 05:22:44 01/08/13 Pacific Standard Time                       
        # 05:22:44 01/08/13 Pacific Standard Time        
    #</methodize4xi>
    
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "send_bytes-of-PipeConnection4Xi" usage = "send_bytes PipeConnection4Xi with (self, buf)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def _send_bytes(self, buf):
        '''
        send_bytes PipeConnection4Xi
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4)            
        # -----------------------------------------------------------------------------------------
        class2print = "PipeConnection4Xi"
        method2print = "_send_bytes"
        
        message2print = "{(Step 1/4a) started" + "; send_bytes-of-PipeConnection4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # time: 04:57:16 01/08/13 Pacific Standard Time
        # time: 05:22:44 01/08/13 Pacific Standard Time
        
        message2print = "{(Step 1/4b) passed" + "; buf-to-send_bytes-of-PipeConnection4Xi: " + str(buf) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # <memory at 0x0000000003979430>   
        # <memory at 0x0000000003687430> 
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/4)            
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 2/4a) passing" + "; handle-passing-to-winapi-WriteFile-by-send_bytes-of-PipeConnection4Xi: " + str(self._handle) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 256; 256
        
        message2print = "{(Step 2/4b) passing" + "; buf-passing-to-winapi-WriteFile-by-send_bytes-of-PipeConnection4Xi: " + str(buf) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # <memory at 0x0000000003979430>
        # <memory at 0x0000000003687430>
        
         
        ov2xi = None
        err2xi = None
        try:
            ov2xi, err2xi = _winapi.WriteFile(self._handle, buf, overlapped=True)
            
            message2print = "{(Step 2/4d) performed" + "; WriteFile-performed-by-send_bytes-of-PipeConnection4Xi: " + str(_winapi.WriteFile)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <built-in function WriteFile>
            
            message2print = "{(Step 2/4e) got" + "; ov2xi-got-by-send_bytes-of-PipeConnection4Xi: " + str(ov2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <_winapi.Overlapped object at 0x0000000003676160>
            
            message2print = "{(Step 2/4f) got" + "; err2xi-got-by-send_bytes-of-PipeConnection4Xi: " + str(err2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 0
        except:
            try:
                message2print = "{(Step 2/4z1) failed" + "; ov2xi-failed-by-send_bytes-of-PipeConnection4Xi: " + str(ov2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                message2print = "{(Step 2/4z1) failed" + "; err2xi-failed-by-send_bytes-of-PipeConnection4Xi: " + str(err2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                message2print = "{(Step 2/4z3) failed" + "; WriteFile-failed-by-send_bytes-of-PipeConnection4Xi: " + str(_winapi.WriteFile)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                message2print = "{(Step 2/4z4) failed" + "; buf-failed-by-send_bytes-of-PipeConnection4Xi: " + str(buf)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                message2print = "{(Step 2/4z5) failed" + "; handle-failed-by-send_bytes-of-PipeConnection4Xi: " + str(self._handle)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                message2print = "{(Step 2/4z6) failed" + "; overlapped-failed-by-send_bytes-of-PipeConnection4Xi: " + str(overlapped)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            except:
                pass
            
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/4)            
        # -----------------------------------------------------------------------------------------          
        try:
            if err2xi == _winapi.ERROR_IO_PENDING:
                
                waitres2xi = _winapi.WaitForMultipleObjects([ov.event], False, INFINITE)
                message2print = "{(Step 3/4a) got" + "; err2xi-got-by-send_bytes-of-PipeConnection4Xi: " + str(err2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                assert waitres2xi == WAIT_OBJECT_0
        except:
            ov2xi.cancel()
            message2print = "{(Step 3/4z1) failed" + "; ov2xi-failed-by-send_bytes-of-PipeConnection4Xi: " + str(ov2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            message2print = "{(Step 3/4z2) failed" + "; err2xi-failed-by-send_bytes-of-PipeConnection4Xi: " + str(err2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            raise
        finally:
            nwritten2xi, err2xi = ov2xi.GetOverlappedResult(True)
            message2print = "{(Step 3/4a) got" + "; nwritten2xi-got-by-send_bytes-of-PipeConnection4Xi: " + str(nwritten2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 79; 60; 81
             
            message2print = "{(Step 3/4b) got" + "; err2xi-got-by-send_bytes-of-PipeConnection4Xi: " + str(err2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 0
            
        assert err2xi == 0
        message2print = "{(Step 3/4c) asserted" + "; err2xi-asserted-by-send_bytes-of-PipeConnection4Xi: " + str(err2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        
        assert nwritten2xi == len(buf)    
        
        message2print = "{(Step 3/4d) asserted" + "; nwritten2xi-asserted-by-send_bytes-of-PipeConnection4Xi: " + str(nwritten2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 79; 60; 81
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/4)            
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 4/4) returning" + "; send_bytes-of-PipeConnection4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 05:22:44 01/08/13 Pacific Standard Time                       
        # 05:22:44 01/08/13 Pacific Standard Time
        # 05:22:44 01/08/13 Pacific Standard Time
                
    #</methodize4xi>
                    
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "recv_bytes-of-PipeConnection4Xi" usage = "recv_bytes PipeConnection4Xi with (self, maxsize)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def _recv_bytes(self, maxsize=None):
        '''
        recv_bytes PipeConnection4Xi
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4)            
        # -----------------------------------------------------------------------------------------
        class2print = "PipeConnection4Xi"
        method2print = "_recv_bytes"
        
        message2print = "{(Step 1/4a) started" + "; recv_bytes-of-PipeConnection4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 05:22:44 01/08/13 Pacific Standard Time
        # 05:22:44 01/08/13 Pacific Standard Time
        
        message2print = "{(Step 1/4b) passed" + "; maxsize-to-recv_bytes-of-PipeConnection4Xi: " + str(maxsize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # None 
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/4)            
        # -----------------------------------------------------------------------------------------                 
        if self._got_empty_message:
            self._got_empty_message = False
            
            message2print = "{(Step 2/4a) set" + "; got_empty_message-set-by-recv_bytes-of-PipeConnection4Xi to: " + str(self._got_empty_message)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            message2print = "{(Step 2/4b) returning" + "; recv_bytes-of-PipeConnection4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #    
            return io.BytesIO()
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/4)            
        # -----------------------------------------------------------------------------------------          
        else:
            bsize2xi = 128 if maxsize is None else min(maxsize, 128)
            message2print = "{(Step 3/4a) set" + "; bsize2xi-set-by-recv_bytes-of-PipeConnection4Xi to: " + str(bsize2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 128
            
            ov2xi = None
            err2xi = None
            
            try:
                ov2xi, err2xi = _winapi.ReadFile(self._handle, bsize2xi, overlapped=True)
                message2print = "{(Step 3/4b) got" + "; ov2xi-got-by-recv_bytes-of-PipeConnection4Xi: " + str(ov2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <_winapi.Overlapped object at 0x0000000003676DD8>
                
                message2print = "{(Step 3/4c) got" + "; err2xi-got-by-recv_bytes-of-PipeConnection4Xi: " + str(err2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 0
                
                try:
                    if err2xi == _winapi.ERROR_IO_PENDING:
                        message2print = "{(Step 3/4d1) calling" + "; WaitForMultipleObjects-calling-by-recv_bytes-of-PipeConnection4Xi: " + str(_winapi.WaitForMultipleObjects)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                         
                        waitres2xi = _winapi.WaitForMultipleObjects([ov2xi.event], False, INFINITE)
                        
                        message2print = "{(Step 3/4d2) got" + "; waitres2xi-got-by-recv_bytes-of-PipeConnection4Xi: " + str(waitres2xi)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                        
                        assert waitres2xi == WAIT_OBJECT_0
                except:
                    try:
                        message2print = "{(Step 3/4z1) failed" + "; ov2xi-failed-by-recv_bytes-of-PipeConnection4Xi: " + str(ov2xi)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                        
                        ov2xi.cancel()
                        raise
                    except:
                        pass    
                finally:
                    message2print = "{(Step 3/4d1) calling" + "; GetOverlappedResult-calling-by-recv_bytes-of-PipeConnection4Xi: " + str(ov2xi.GetOverlappedResult)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # <built-in method GetOverlappedResult of _winapi.Overlapped object at 0x0000000003676DD8>
                    
                    nread2xi, err2xi = ov2xi.GetOverlappedResult(True)
                    
                    message2print = "{(Step 3/4e2) got" + "; nread2xi-got-by-recv_bytes-of-PipeConnection4Xi: " + str(nread2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 79; 20; 81 
                    
                    message2print = "{(Step 3/4f) got" + "; err2xi-got-by-recv_bytes-of-PipeConnection4Xi: " + str(err2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 0
                    
                    if err2xi == 0:
                        f2xi = io.BytesIO()
                        message2print = "{(Step 3/4g) got" + "; f2xi-got-by-recv_bytes-of-PipeConnection4Xi: " + str(f2xi)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # <_io.BytesIO object at 0x0000000003580FA8>
                        
                        buffer2xi = ov2xi.getbuffer()
                        message2print = "{(Step 3/4h) got" + "; buffer2xi-got-by-recv_bytes-of-PipeConnection4Xi: " + str(buffer2xi)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # b'\x80\x03]q\x00(K\x01X=\x00\x00\x00(STEP 2A/33) OF HELLO DUPLEX PIPE4XI: HELLO B4XI, A4XI HERE !q\x01Ne.'
                        # b'(STEP 4/33) OF HELLO DUPLEX PIPE4XI: ROGER A4XI, B4XI HERE !'
                        # b'\x00\x00\x00\x00\x01\x00\x00\x00\x02\x00\x00\x00\x03\x00\x00\x00\x04\x00\x00\x00'
                        # b'(STEP 8A/33) OF HELLO DUPLEX PIPE4XI: ARRAY RECEIVED OK, ROGER FROM B4XI TO A4XI!'
                        
                        f2xi.write(buffer2xi)
                        
                        message2print = "{(Step 3/4i) performed" + "; write-performed-by-recv_bytes-of-PipeConnection4Xi: " + str(f2xi.write)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # <built-in method write of _io.BytesIO object at 0x0000000003580FA8>
                        
                        message2print = "{(Step 3/4j) returning" + "; f2xi-returning-by-recv_bytes-of-PipeConnection4Xi: " + str(f2xi)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # <_io.BytesIO object at 0x0000000003580FA8>
                        
                        return f2xi
                    elif err2xi == _winapi.ERROR_MORE_DATA:
                    
                        message2print = "{(Step 3/4k) passing" + "; ov2xi-passing-to-get_more_data-by-recv_bytes-of-PipeConnection4Xi: " + str(ov2xi) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        
                        message2print = "{(Step 3/4l) passing" + "; maxsize-passing-to-get_more_data-by-recv_bytes-of-PipeConnection4Xi: " + str(maxsize) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                    
                        message2print = "{(Step 3/4m) returning" + "; get_more_data-returning-by-recv_bytes-of-PipeConnection4Xi: " + str(self._get_more_data)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                        
                        return self._get_more_data(ov2xi, maxsize)
            except IOError as e:
                
                if e.winerror == _winapi.ERROR_BROKEN_PIPE:
                    message2print = "{(Step 3/4z2) failed" + "; winerror-failed-by-recv_bytes-of-PipeConnection4Xi: " + str(e.winerror)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                
                    raise EOFError
                else:
                    message2print = "{(Step 3/4z3) failed" + "; error-failed-by-recv_bytes-of-PipeConnection4Xi: " + str(e)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    raise
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/4)            
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 4/4z1) failed" + "; error-failed-by-recv_bytes-of-PipeConnection4Xi: " + str("recv_bytes-of-PipeConnection4Xi shouldn't get here")  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        raise RuntimeError("recv_bytes-of-PipeConnection4Xi shouldn't get here; expected KeyboardInterrupt")
        

    #</methodize4xi>
                

            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "poll-of-PipeConnection4Xi" usage = "poll PipeConnection4Xi with (self, timeout)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def _poll(self, timeout):
        '''
        poll PipeConnection4Xi
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/2)            
        # -----------------------------------------------------------------------------------------
        class2print = "PipeConnection4Xi"
        method2print = "_poll"
        
        message2print = "{(Step 1/3a) started" + "; poll-of-PipeConnection4Xi-started: " + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        #   
        
        message2print = "{(Step 1/3b) passed" + "; timeout-to-poll-of-PipeConnection4Xi: " + str(timeout) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #  
        
        answer2poll = False
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/3)            
        # -----------------------------------------------------------------------------------------                 
        if (self._got_empty_message or
            _winapi.PeekNamedPipe(self._handle)[0] != 0):
            return True

        # -----------------------------------------------------------------------------------------
        # (Step 3/3)            
        # -----------------------------------------------------------------------------------------           
        return bool(wait([self], timeout))
                            
    #</methodize4xi>
                        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get_more_data-of-PipeConnection4Xi" usage = "get_more_data PipeConnection4Xi with (self, ov, maxsize)">
    #<note4xi note = "new in [Rel 17.21]"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting [Rel 17.21]"/>
    # ---------------------------------------------------------------------------------------------
    def _get_more_data(self, ov, maxsize):
        '''
        get_more_data PipeConnection4Xi
        '''        
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/4)            
        # -----------------------------------------------------------------------------------------
        class2print = "PipeConnection4Xi"
        method2print = "_get_more_data"
        
        message2print = "{(Step 1/4a) started" + "; get_more_data-of-PipeConnection4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 04:31:47 02/19/13 Pacific Standard Time
        
        message2print = "{(Step 1/4b) passed" + "; ov-to-get_more_data-of-PipeConnection4Xi: " + str(ov) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # <_winapi.Overlapped object at 0x00000000031FFC10> 
        
        message2print = "{(Step 1/4c) passed" + "; maxsize-to-get_more_data-of-PipeConnection4Xi: " + str(maxsize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # None 
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/4)            
        # -----------------------------------------------------------------------------------------                 
        buf2xi = None
        try:
            buf2xi = ov.getbuffer()
            message2print = "{(Step 2/4a) got" + "; buf2xi-got-by-get_more_data-of-PipeConnection4Xi: " + str(buf2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # b'\x80\x03]q\x00(X\x0c\x00\x00\x00shot2ball4xiq\x01}q\x02X\x06\x00\x00\x000x0y0zq\x03}q\x04(X\t\x00\x00\x00verb2ballq\x05X\x0b\x00\x00
            #   \x00skeletonizeq\x06X\x06\x00\x00\x00y2ballq\x07K\x00X\x06\x00\x00\x00x2ballq\x08K\x00X\x06\x00\x00\x00z2ballq\tK\x00X\n\x00\x00'
            # ; time: 04:31:47 02/19/13 Pacific Standard Time}

        except:
            message2print = "{(Step 2/4z1) failed" + "; buf2xi-failed-by-get_more_data-of-PipeConnection4Xi: " + str(buf2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        f2xi = None
        try:    
            f2xi = io.BytesIO()
            message2print = "{(Step 2/4b) got" + "; f2xi-got-by-get_more_data-of-PipeConnection4Xi: " + str(f2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <_io.BytesIO object at 0x000000000359DEE8>
        except:
            message2print = "{(Step 2/4z2) failed" + "; f2xi-failed-by-get_more_data-of-PipeConnection4Xi: " + str(f2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #         
            
        message2print = "{(Step 2/4c) passing" + "; buf2xi-passing-to-write-by-get_more_data-of-PipeConnection4Xi: " + str(buf2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # b'\x80\x03]q\x00(X\x0c\x00\x00\x00shot2ball4xiq\x01}q\x02X\x06\x00\x00\x000x0y0zq\x03}q\x04(X\t\x00\x00\x00verb2ballq\x05X\x0b\x00\x00
        #   \x00skeletonizeq\x06X\x06\x00\x00\x00y2ballq\x07K\x00X\x06\x00\x00\x00x2ballq\x08K\x00X\x06\x00\x00\x00z2ballq\tK\x00X\n\x00\x00'
        # ; time: 04:31:47 02/19/13 Pacific Standard Time}
        
        try:
            f2xi.write(buf2xi)
            
            message2print = "{(Step 2/4d) performed" + "; write-performed-by-get_more_data-of-PipeConnection4Xi: " + str(f2xi.write)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <built-in method write of _io.BytesIO object at 0x000000000359DEE8>    
        except:
            message2print = "{(Step 2/4z3) failed" + "; f2xi-failed-by-get_more_data-of-PipeConnection4Xi: " + str(f2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #    
            
        # -----------------------------------------------------------------------------------------
        # (Step 3/4)            
        # -----------------------------------------------------------------------------------------          
        message2print = "{(Step 3/4a) passing" + "; handle-passing-to-_winapi.PeekNamedPipe-by-get_more_data-of-PipeConnection4Xi: " + str(self._handle) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 412
        left2xi = 0
        try:
            
            left2xi = _winapi.PeekNamedPipe(self._handle)[1]
            message2print = "{(Step 3/4b) got" + "; left2xi-got-by-get_more_data-of-PipeConnection4Xi: " + str(left2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 47
        except:
            message2print = "{(Step 3/4z1) failed" + "; left2xi-failed-by-get_more_data-of-PipeConnection4Xi: " + str(left2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        assert left2xi > 0
        
        if maxsize is not None and len(buf2xi) + left2xi > maxsize:
        
            self._bad_message_length()
            
            try:
                message2print = "{(Step 3/4z1) warned" + "; buf2xi-warned-by-get_more_data-of-PipeConnection4Xi: " + str(buf2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                message2print = "{(Step 3/4z2) warned" + "; left2xi-warned-by-get_more_data-of-PipeConnection4Xi: " + str(left2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                message2print = "{(Step 3/4z3) warned" + "; maxsize-warned-by-get_more_data-of-PipeConnection4Xi: " + str(maxsize)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                
                message2print = "{(Step 3/4z4) performed" + "; bad_message_length-performed-by-get_more_data-of-PipeConnection4Xi: " + str(self._bad_message_length)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            except:
                pass
                   
        ov2xi = None
        err2xi = None
        rbytes2xi = None
        
        message2print = "{(Step 3/4c1) passing" + "; handle-passing-to-_winapi.ReadFile: " + str(self._handle) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 412
        message2print = "{(Step 3/4c2) passing" + "; left2xi-passing-to-_winapi.ReadFile: " + str(left2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 47

        try:
            ov2xi, err2xi = _winapi.ReadFile(self._handle, left2xi, overlapped=True)
                        
            message2print = "{(Step 3/4d1) got" + "; ov2xi-got-by-get_more_data-of-PipeConnection4Xi: " + str(ov2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            message2print = "{(Step 3/4d2) got" + "; err2xi-got-by-get_more_data-of-PipeConnection4Xi: " + str(err2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        except:
            message2print = "{(Step 3/4z5) failed" + "; ov2xi-failed-by-get_more_data-of-PipeConnection4Xi: " + str(ov2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        message2print = "{(Step 3/4e1) performing" + "; GetOverlappedResult-performing-by-get_more_data-of-PipeConnection4Xi: " + str(ov2xi.GetOverlappedResult) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #                 
        try:    
            rbytes2xi, err2xi = ov2xi.GetOverlappedResult(True)
            
            message2print = "{(Step 3/4e2) got" + "; rbytes2xi-got-by-get_more_data-of-PipeConnection4Xi: " + str(rbytes2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            message2print = "{(Step 3/4e3) got" + "; err2xi-got-by-get_more_data-of-PipeConnection4Xi: " + str(err2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        except:
            message2print = "{(Step 3/4z6) failed" + "; rbytes2xi-failed-by-get_more_data-of-PipeConnection4Xi: " + str(rbytes2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        assert err2xi == 0
        
        assert rbytes2xi == left2xi
        
        buffer2xi = None
        message2print = "{(Step 3/4h1) performing" + "; getbuffer-performing-by-get_more_data-of-PipeConnection4Xi: " + str(ov2xi.getbuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #    
        try:                
            buffer2xi = ov2xi.getbuffer()
            message2print = "{(Step 3/4h2) got" + "; buffer2xi-got-by-get_more_data-of-PipeConnection4Xi: " + str(buffer2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        except:
            message2print = "{(Step 3/4z7) failed" + "; buffer2xi-failed-by-get_more_data-of-PipeConnection4Xi: " + str(buffer2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #     
        
          
        try:
            message2print = "{(Step 3/4j1) passing" + "; buffer2xi-passing-to-write: " + str(buffer2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            f2xi.write(buffer2xi)
                
            message2print = "{(Step 3/4j2) performed" + "; write-performed-by-get_more_data-of-PipeConnection4Xi: " + str(f2xi.write)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        except:
            message2print = "{(Step 3/4z8) failed" + "; write-failed-by-get_more_data-of-PipeConnection4Xi: " + str(buffer2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/4)            
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 3/4k) returning" + "; f2xi-returning-by-get_more_data-of-PipeConnection4Xi: " + str(f2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #                
        
        return f2xi
                
    #</methodize4xi>
                                                                                                                               
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-PipeConnection4Xi" usage = "senses PipeConnection4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses PipeConnection4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
        
        class2print = 'PipeConnection4Xi'
        method2print = 'sense'
        
           
        message2print = "{(Step 1/5) started" + "; sense-of-PipeConnection4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------  
        message2print = "{(Step 2/5) sensing" + "; PipeConnection4Xi-sensing-by-sense-of-PipeConnection4Xi: " + str(PipeConnection4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                  
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3/5) got" + "; knowledge-base2xi-got-by-sense-of-PipeConnection4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            
        
        
        # -----------------------------------------------------------------------------------------
        # (Step 4/5) add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
    
        
        
      
        message2print = "{(Step 4/5) sensed" + "; PipeConnection4Xi-sensed-by-sense-of-PipeConnection4Xi: " + str(PipeConnection4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------         
        message2print = "{(Step 5/5) returning" + "; knowledge-base2xi-returning-by-sense-of-PipeConnection4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
          
        return kb2xi
        
    #</methodize4xi>
      

        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-PipeConnection4Xi" usage = "reasons PipeConnection4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons PipeConnection4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
             
        
        class2print = 'PipeConnection4Xi'
        method2print = 'reason'

           
        message2print = "{(Step 1/5) started" + "; reason-of-PipeConnection4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2/5) reasoning" + "; PipeConnection4Xi-reasoning-by-reason-of-PipeConnection4Xi: " + str(PipeConnection4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
            
        
        # -----------------------------------------------------------------------------------------
        #  (Step 3/5) get reasoned knowledge for this object -  Non Windows
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        message2print = "{(Step 3/5) got" + "; knowledge-base4xi-got-by-by-reason-of-PipeConnection4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            
        
        # -----------------------------------------------------------------------------------------
        #  (Step 4/5) add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        
        message2print = "{(Step 4/5) reasoning" + "; PipeConnection4Xi-reasoning-by-reason-of-PipeConnection4Xi: " + str(PipeConnection4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 5/5) returning" + "; knowledge-base4xi-returning-by-reason-of-PipeConnection4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                          
          
        return kb4xi
        
    #</methodize4xi>
         

    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-PipeConnection4Xi" usage = "aces PipeConnection4Xi class objects">
    #<note4xi note = [Rel 7.21]""/>
    #<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces PipeConnection4Xi class objects 
        '''
          
        # -----------------------------------------------------------------------------------------
        # (Step 1/5)      
        # -----------------------------------------------------------------------------------------
            
        
        class2print = 'PipeConnection4Xi'
        method2print = 'ace'
        
           
        message2print = "{(Step 1/5) started" + "; ace-of-PipeConnection4Xi-OSx-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        # (Step 2/5) -  Non Windows
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2/5) acing" + "; PipeConnection4Xi-acing-by-ace-of-PipeConnection4Xi: " + str(PipeConnection4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
            
        
        # -----------------------------------------------------------------------------------------
        # (Step 3/5) get aced knowledge for this object -  Non Windows
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3/5)got" + "; knowledge-base8xi-got-by-ace-of-PipeConnection4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            


        # -----------------------------------------------------------------------------------------
        # (Step 4/5) add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{(Step 4/5) acing" + "; PipeConnection4Xi-acing-by-ace-of-PipeConnection4Xi: " + str(PipeConnection4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
        # -----------------------------------------------------------------------------------------
        # (Step 5/5) -  Non Windows
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 5/5) returning" + "; knowledge-base8xi-returning-by-ace-of-PipeConnection4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
        
        return kb8xi
    #</methodize4xi>
#</class4xi>


# ====================================================================================================================
# REGISTER
#====================================================================================================================
if sys.platform == 'win32':
    ForkingPickler4Xi.register(socket.socket, reduction.reduce_socket)
    ForkingPickler4Xi.register(PipeConnection4Xi, reduction.reduce_connection)
   
else:
    ForkingPickler4Xi.register(socket.socket, reduction.reduce_socket)
    ForkingPickler4Xi.register(PipeConnection4Xi, reduction.reduce_connection)
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of pipeconnection4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

